#include <QFile>
#include <QDateTime>
#include <QMessageBox>
#include <QTextStream>

#include "Info.h"

extern Player g_voice;

Info::Info(QWidget *parent) : CenterFrame(parent) {
   // Get the date:
   QDateTime t;
   QDate date=t.currentDateTime().date();
   QString sDay,sMonth,sYear,sDate;
   sDay.setNum(date.day());
   sMonth.setNum(date.month());
   sYear.setNum(date.year());
   sDate=sDay + tr("/") + sMonth + tr("-") + sYear;

   lo=new QVBoxLayout;
   loForm=new QGridLayout;
   loButtons=new QHBoxLayout;

   laHeadLine=new QLabel(tr("<html><b>Oplysninger om dig selv</b></html>"));
   laHeadLine->setFont(foHeadLine);

   la1=new QLabel(tr("<html><hr>Her skal du skrive lidt oplysninger om dig selv.</html>"));
   la2=new QLabel(tr("<html>Du <u>skal</u> udfylde de <font color=\"orange\">orange</font> felter!<hr>"));

   // Form:
   la3=new QLabel(tr("Dato:"));
   leDate=new QLineEdit;
   leDate->setMinimumSize(350,0); // Makes the whole form a little bit wider
   leDate->setText(sDate);

   la4=new QLabel(tr("Undervisningssted:"));
   lePlace=new QLineEdit;

   la5=new QLabel(tr("<html><font color=\"orange\">Navn:</font></html>"));
   leName=new QLineEdit;

   la6=new QLabel(tr("<html><font color=\"orange\">Adresse:</font></html>"));
   leAddress=new QLineEdit;

   la7=new QLabel(tr("<html><font color=\"orange\">Fødselsår:</font></html>"));
   leCPR=new QLineEdit;

   la8=new QLabel(tr("Hvor længe har du gået i folkeskole?"));
   leYears=new QLineEdit;

   loForm->addWidget(la3,0,0,Qt::AlignRight);
   loForm->addWidget(leDate,0,1);
   loForm->addWidget(la4,1,0,Qt::AlignRight);
   loForm->addWidget(lePlace,1,1);
   loForm->addWidget(la5,2,0,Qt::AlignRight);
   loForm->addWidget(leName,2,1);
   loForm->addWidget(la6,3,0,Qt::AlignRight);
   loForm->addWidget(leAddress,3,1);
   loForm->addWidget(la7,4,0,Qt::AlignRight);
   loForm->addWidget(leCPR,4,1);
   loForm->addWidget(la8,5,0,Qt::AlignRight);
   loForm->addWidget(leYears,5,1);

   la9=new QLabel(tr("Har du studentereksamen eller tilsvarende?"));
   leHighschool=new QLineEdit;

   la10=new QLabel(tr("Har du en uddannelse? Hvilken?"));
   leEducation=new QLineEdit;

   la11=new QLabel(tr("Er du i job? Hvilket?"));
   leJob=new QLineEdit;

   la12=new QLabel(tr("<html><hr></html>"));

   // Button group:
   dbVoice=new DarkButton(this,tr("voice.png"));
   dbNext=new DarkButton(this,tr("next.png"));
   loButtons->addWidget(dbVoice);
   loButtons->addStretch();
   loButtons->addWidget(dbNext);

   // Entire layout:
   lo->addSpacing(20);
   lo->addWidget(laHeadLine);
   lo->addSpacing(20);
   lo->addWidget(la1);
   lo->addWidget(la2);
   lo->addSpacing(10);
   lo->addLayout(loForm);
   lo->addSpacing(20);
   lo->addWidget(la9);
   lo->addWidget(leHighschool);
   lo->addWidget(la10);
   lo->addWidget(leEducation);
   lo->addWidget(la11);
   lo->addWidget(leJob);
   lo->addWidget(la12);
   lo->addLayout(loButtons);
   lo->addStretch();
   setLayout(lo);

   // Handle returns:
   connect(leDate,SIGNAL(returnPressed()),lePlace,SLOT(setFocus()));
   connect(lePlace,SIGNAL(returnPressed()),leName,SLOT(setFocus()));
   connect(leName,SIGNAL(returnPressed()),leAddress,SLOT(setFocus()));
   connect(leAddress,SIGNAL(returnPressed()),leCPR,SLOT(setFocus()));
   connect(leCPR,SIGNAL(returnPressed()),leYears,SLOT(setFocus()));
   connect(leYears,SIGNAL(returnPressed()),leHighschool,SLOT(setFocus()));
   connect(leHighschool,SIGNAL(returnPressed()),leEducation,SLOT(setFocus()));
   connect(leEducation,SIGNAL(returnPressed()),leJob,SLOT(setFocus()));
   connect(leJob,SIGNAL(returnPressed()),lePlace,SLOT(setFocus()));

   connect(dbVoice,SIGNAL(clicked()),this,SLOT(slotPlayVoice()));
   connect(dbNext,SIGNAL(clicked()),this,SLOT(slotNextPage()));
}

void Info::saveInfo() {
   QFile file("results.html");
   if (!file.open(QIODevice::WriteOnly)) {
      QMessageBox::critical(this,tr("Kritisk fejl!"),tr("Programmet kan ikke skrive til\nfilen \"results.html\".\n\nKontakt din lærer!"),QMessageBox::Ok);
      exit(1);
   }
   QTextStream stream(&file);
   stream.setCodec("UTF-8");

   stream << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n\n<html>\n\n";
   stream << "<head>\n";
   stream << "<title>Resultater</title>\n";
   stream << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n";
   stream << "</head>\n\n";
   stream << "<body>\n\n";
   stream << "<h2><u>Diktat til trinindplacering</u></h2>\n\n";
   stream << "<p>\n<small>\n";
   stream << "Dato: " << leDate->text() << "<br>\n";
   stream << "Undervisningssted: " << lePlace->text() << "<br>\n";
   stream << "Navn: " << leName->text() << "<br>\n";
   stream << "Adresse: " << leAddress->text() << "<br>\n";
   stream << tr("Fødselsår: ") << leCPR->text() << "<br>\n";
   stream << tr("Antal år i folkeskole: ") << leYears->text() << "<br>\n";
   stream << "Studentereksamen: " << leHighschool->text() << "<br>\n";
   stream << "Uddannelse: " << leEducation->text() << "<br>\n";
   stream << "Jobs: " << leJob->text() << "\n</small>\n\n";

   stream.flush();
   file.close();
}

void Info::slotNextPage() {
   // Validation:
   if (leName->text().isEmpty()) {
      QMessageBox::warning(this,tr("Ups!"),tr("Du mangler at skrive dit navn!"),QMessageBox::Ok);
      leName->setFocus();
   }
   else if (leAddress->text().isEmpty()) {
      QMessageBox::warning(this,tr("Ups!"),tr("Du mangler at skrive din adresse!"),QMessageBox::Ok);
      leAddress->setFocus();
   }
   else if (leCPR->text().isEmpty()) {
      QMessageBox::warning(this,tr("Ups!"),tr("Du mangler at skrive dit fødselsår!"),QMessageBox::Ok);
      leCPR->setFocus();
   }
   else {
      // Save information:
      saveInfo();
      g_voice.stopSound();
      emit nextPage(); // Sends next-page-button clicked signal, grabbed by MainWindow.
   }
}

void Info::slotPlayVoice() {
   g_voice.playSound(tr("sounds/info.wav"));
}

void Info::showEvent(QShowEvent *e) { // Sets the initial focus to the place of education
   activateWindow();
   lePlace->setFocus();
   g_voice.playSound(tr("sounds/info.wav"));
   QWidget::showEvent(e);
}
